# Demo of flaw in Pythn 2.7.3's mimetypes.read_windows_registry() method, with demo of fixed method
# 27 June 2012 
# Dave Chambers 
# dlchambers@aol.com

import os
import sys
import _winreg
import mimetypes_fixed as mtf
import mimetypes


default_encoding = sys.getdefaultencoding()

# Class method copied from mimetypes.py
def enum_types(mimedb):
    i = 0
    while True:
        try:
            ctype = _winreg.EnumKey(mimedb, i)
        except EnvironmentError:
            break
        try:
            ctype = ctype.encode(default_encoding) # omit in 3.x!
        except UnicodeEncodeError:
            pass
        else:
            yield ctype
        i += 1


# Excerpt of mimetypes.read_windows_registry()
# This method is flawed because it uses the wrong registry key.
# HKCR\MIME\Database\Content Type should be used for *reverse lookups*: Use it to get the files extension for a known MIME type.
# The bug happens when >1 MIME types are mapped to a particular extension.
# For example, both "image/png" and "image/x-png" map to the ".png" file extension.
# Unfortunately, what happens is this code finds "image/png", then later finds "image/x-png" and this steals the ".png" extension.
# For a proper lookup, see dave_fix() below
def current_flawed():        
    with _winreg.OpenKey(_winreg.HKEY_CLASSES_ROOT, r'MIME\Database\Content Type') as mimedb:
        for ctype in enum_types(mimedb):
            try:
                with _winreg.OpenKey(mimedb, ctype) as key:
                    suffix, datatype = _winreg.QueryValueEx(key, 'Extension')
            except EnvironmentError:
                continue
            if datatype != _winreg.REG_SZ:
                continue
            try:
                suffix = suffix.encode(default_encoding) # omit in 3.x!
            except UnicodeEncodeError:
                continue
                
            if suffix == '.png':
                print "   Interpret {} as {}".format(suffix, ctype)


# This code uses the correct regkey.
# The HKCR root contains extension-to-mimetype mappings.
# What we do is enum the HKCR root, find all subkeys that start with a dot, then inspect those for a 'Content Type' value.
# We will ignore many subkeys (undotted and those w/o a 'Content Type' value), but this is the correct procedure under Windows.
def dave_fix():
    
    with _winreg.OpenKey(_winreg.HKEY_CLASSES_ROOT, "") as hkcr:
        for subkeyname in enum_types(hkcr):
            try:
                with _winreg.OpenKey(hkcr, subkeyname) as subkey:
                    if subkeyname[0] == ".":    # Only check entries that begin with dots, i.e. file extensions
                        # subkeyname is the file extension
                        mimetype, datatype = _winreg.QueryValueEx(subkey, 'Content Type')  # throws EnvironmentError if no 'Content Type' value
                        if datatype != _winreg.REG_SZ:
                            continue
                            
                        try:
                            subkeyname = subkeyname.encode(default_encoding) # omit in 3.x!
                        except UnicodeEncodeError:
                            continue
                            
                        if subkeyname == '.png':
                            print "   Interpret {} as {}".format(subkeyname, mimetype)
                        
            except EnvironmentError:
                continue
    

print "Flawed method (notice 2 mappings, and image/x-png overwrites image/png):"
current_flawed()
print
print "Fixed method (note the single, correct mapping of .png to 'image/png'):"
dave_fix()
print
print "Module calls:"
print "   flawed: {}".format(mimetypes.guess_type('test.png'))
print "   fixed: {}".format(mtf.guess_type('test.png'))
